@@ -1,6 +1,6 @@
 /* the core of phase vocoder with complex arithmetics
  * Copyright (C) 2007 Kengo Ichiki <kichiki@users.sourceforge.net>
- * $Id: pv-complex.c,v 1.10 2007/03/11 01:15:13 kichiki Exp $
+ * $Id: pv-complex.c,v 1.15 2007/11/04 23:57:06 kichiki Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include "memory-check.h" // CHECK_MALLOC() macro
 
 // FFTW library
 #include <fftw3.h>
@@ -37,20 +38,18 @@
 // samplerate
 #include <samplerate.h>
 
-#include "memory-check.h" // CHECK_MALLOC() macro
 
-#include "pv-conventional.h" // pv_play_resample() for check
+#include "pv-conventional.h" // get_scale_factor_for_window()
 #include "pv-complex.h"
 
 
 /** utility routines for struct pv_omplex_data **/
 
-struct pv_complex_data *
+struct pv_complex *
 pv_complex_init (long len, long hop_syn, int flag_window)
 {
-  struct pv_complex_data *pv = NULL;
-
-  pv = (struct pv_complex_data *) malloc (sizeof (struct pv_complex_data));
+  struct pv_complex *pv
+    = (struct pv_complex *) malloc (sizeof (struct pv_complex));
   CHECK_MALLOC (pv, "pv_complex_init");
 
   pv->len = len;
@@ -100,8 +99,29 @@ pv_complex_init (long len, long hop_syn, int flag_window)
   return (pv);
 }
 
+/* change rate and pitch (note that hop_syn is fixed)
+ * INPUT
+ *  pv : struct pv_complex
+ *  rate  : rate of speed (1 == same speed, negative == backward)
+ *  pitch : pitch-shift (0 == no-shift, +1(-1) = half-note up(down))
+ * OUTPUT
+ *  pv->hop_res : hop size in 1 step for the output
+ *                == hop_syn with no pitch-shifting
+ *  pv->hop_ana : hop size in 1 step for the input
+ *                == hop_syn with no time-stretching (rate = 1).
+ */
 void
-pv_complex_set_input (struct pv_complex_data *pv,
+pv_complex_change_rate_pitch (struct pv_complex *pv,
+			      double rate,
+			      double pitch)
+{
+  pv->hop_res = (long)((double)pv->hop_syn * pow (2.0, - pitch / 12.0));
+  pv->hop_ana = (long)((double)pv->hop_res * rate);
+}
+
+
+void
+pv_complex_set_input (struct pv_complex *pv,
 		      SNDFILE *sf, SF_INFO *sfinfo)
 {
   pv->sf = sf;
@@ -109,7 +129,7 @@ pv_complex_set_input (struct pv_complex_data *pv,
 }
 
 void
-pv_complex_set_output_ao (struct pv_complex_data *pv,
+pv_complex_set_output_ao (struct pv_complex *pv,
 			  ao_device *ao)
 {
   pv->flag_out = 0;
@@ -117,7 +137,7 @@ pv_complex_set_output_ao (struct pv_complex_data *pv,
 }
 
 void
-pv_complex_set_output_sf (struct pv_complex_data *pv,
+pv_complex_set_output_sf (struct pv_complex *pv,
 			  SNDFILE *sf, SF_INFO *sfinfo)
 {
   pv->flag_out = 1;
@@ -126,7 +146,7 @@ pv_complex_set_output_sf (struct pv_complex_data *pv,
 }
 
 void
-pv_complex_free (struct pv_complex_data *pv)
+pv_complex_free (struct pv_complex *pv)
 {
   if (pv == NULL) return;
 
@@ -148,34 +168,33 @@ pv_complex_free (struct pv_complex_data *pv)
 }
 
 
-static long
-read_and_FFT_stereo (struct pv_complex_data *pv,
+long
+read_and_FFT_stereo (struct pv_complex *pv,
 		     long frame,
 		     double *f_left, double *f_right)
 {
-  int i;
-
-  double * left = NULL;
-  double * right = NULL;
-  left  = (double *) malloc (sizeof (double) * pv->len);
-  right = (double *) malloc (sizeof (double) * pv->len);
-  CHECK_MALLOC (left,  "read_and_FFT_stereo");
-  CHECK_MALLOC (right, "read_and_FFT_stereo");
+  static double *left  = NULL;
+  static double *right = NULL;
+  if (left == NULL)
+    {
+      left  = (double *)malloc (sizeof (double) * pv->len);
+      right = (double *)malloc (sizeof (double) * pv->len);
+      CHECK_MALLOC (left,  "read_and_FFT_stereo");
+      CHECK_MALLOC (right, "read_and_FFT_stereo");
+    }
 
   long status;
   status = sndfile_read_at (pv->sf, *(pv->sfinfo), frame,
 			    left, right, pv->len);
   if (status != pv->len)
     {
-      free (left);
-      free (right);
-
       return (status);
     }
 
   // FFT for left channel
   windowing (pv->len, left, pv->flag_window, 1.0, pv->time);
   fftw_execute (pv->plan); // FFT: time[] -> freq[]
+  int i;
   for (i = 0; i < pv->len; i ++)
     {
       f_left [i] = pv->freq [i];
@@ -189,9 +208,6 @@ read_and_FFT_stereo (struct pv_complex_data *pv,
       f_right [i] = pv->freq [i];
     }
 
-  free (left);
-  free (right);
-
   return (status);
 }
 
@@ -199,8 +215,8 @@ read_and_FFT_stereo (struct pv_complex_data *pv,
  * INPUT
  *  scale : for safety (give 0.5, for example)
  */
-static void
-apply_invFFT_mono (struct pv_complex_data *pv,
+void
+apply_invFFT_mono (struct pv_complex *pv,
 		   const double *f, double scale,
 		   double *out)
 {
@@ -240,172 +256,239 @@ check_zero (int n, const double *x)
 }
 
 
-/* play the segment of pv->[lr]_out[] for pv->hop_syn
- * pv->pitch_shift is taken into account
+/* resample pv->[rl]_out[i] for i = 0 to pv->hop_syn
+ *       to [left,right][i] for i = 0 to pv->hop_res
+ * INPUT
+ * OUTPUT
  */
-int
-pv_complex_play_resample (struct pv_complex_data *pv)
+void
+pv_complex_resample (struct pv_complex *pv,
+		     double *left, double *right)
 {
-  int status = 0;
-  int i;
-
   // samplerate conversion
-  float *fl_in  = NULL;
-  float *fl_out = NULL;
-  double *l_out_src = NULL;
-  double *r_out_src = NULL;
   SRC_DATA srdata;
-
-  if (pv->hop_syn != pv->hop_res)
+  static float  *fl_in  = NULL;
+  static float  *fl_out = NULL;
+  static int hop_syn0 = 0;
+  static int hop_res0 = 0;
+  if (fl_in == NULL)
     {
       fl_in  = (float *)malloc (sizeof (float) * 2 * pv->hop_syn);
       fl_out = (float *)malloc (sizeof (float) * 2 * pv->hop_res);
-      CHECK_MALLOC (fl_in,  "pv_complex_play_resample");
-      CHECK_MALLOC (fl_out, "pv_complex_play_resample");
-
-      srdata.input_frames  = pv->hop_syn;
-      srdata.output_frames = pv->hop_res;
-      srdata.src_ratio = (double)(pv->hop_res) / (double)(pv->hop_syn);
-      srdata.data_in  = fl_in;
-      srdata.data_out = fl_out;
-
-      // samplerate conversion (time fixed)
-      for (i = 0; i < pv->hop_syn; i ++)
-	{
-	  fl_in [i*2 + 0] = (float)(pv->l_out [i]);
-	  fl_in [i*2 + 1] = (float)(pv->r_out [i]);
-	}
-      //status = src_simple (&srdata, SRC_SINC_BEST_QUALITY, 2);
-      status = src_simple (&srdata, SRC_SINC_FASTEST, 2);
-      // this works better
-      if (status != 0)
-	{
-	  fprintf (stderr, "fail to samplerate conversion\n");
-	  exit (1);
-	}
-
-      //l_out_src = (double *)malloc (sizeof (double) * hop_in);
-      //r_out_src = (double *)malloc (sizeof (double) * hop_in);
-      l_out_src = (double *)malloc (sizeof (double) * pv->hop_res);
-      r_out_src = (double *)malloc (sizeof (double) * pv->hop_res);
-      CHECK_MALLOC (l_out_src, "pv_complex_play_resample");
-      CHECK_MALLOC (r_out_src, "pv_complex_play_resample");
-
-      //for (i = 0; i < hop_in; i ++)
-      for (i = 0; i < pv->hop_res; i ++)
-	{
-	  l_out_src [i] = (double)(fl_out [i*2 + 0]);
-	  r_out_src [i] = (double)(fl_out [i*2 + 1]);
-	}
-      free (fl_in);
-      free (fl_out);
-
-      // output
-      if (pv->flag_out == 0)
-	{
-	  //status = ao_write (pv->ao, l_out_src, r_out_src, hop_in);
-	  status = ao_write (pv->ao, l_out_src, r_out_src, pv->hop_res);
-	  status /= 4; // 2 bytes for 2 channels
-	}
-      else if (pv->flag_out == 1)
-	{
-	  /*
-	  status = sndfile_write (pv->sfout, *(pv->sfout_info),
-				  l_out_src, r_out_src, hop_in);
-	  */
-	  status = sndfile_write (pv->sfout, *(pv->sfout_info),
-				  l_out_src, r_out_src, pv->hop_res);
-	}
-      else
-	{
-	  fprintf (stderr, "invalid output device\n");
-	}
-      free (l_out_src);
-      free (r_out_src);
-
-      // modify status
-      if (status == pv->hop_res) status = pv->hop_syn;
-      else status = (int)((double)status
-			  * (double)(pv->hop_syn) / (double)(pv->hop_res));
+      CHECK_MALLOC (fl_in,  "pv_complex_resample");
+      CHECK_MALLOC (fl_out, "pv_complex_resample");
+      hop_syn0 = pv->hop_syn;
+      hop_res0 = pv->hop_res;
     }
   else
     {
-      if (pv->flag_out == 0)
+      if (hop_syn0 < pv->hop_syn)
 	{
-	  status = ao_write (pv->ao, pv->l_out, pv->r_out, pv->hop_syn);
-	  status /= 4; // 2 bytes for 2 channels
+	  fl_in = (float *)realloc (fl_in, sizeof (float) * 2 * pv->hop_syn);
+	  CHECK_MALLOC (fl_in,  "pv_complex_resample");
+	  hop_syn0 = pv->hop_syn;
 	}
-      else if (pv->flag_out == 1)
+      if (hop_res0 < pv->hop_res)
 	{
-	  status = sndfile_write (pv->sfout, *(pv->sfout_info),
-				  pv->l_out, pv->r_out, pv->hop_syn);
-	}
-      else
-	{
-	  fprintf (stderr, "invalid output device\n");
+	  fl_out =
+	    (float *)realloc (fl_out, sizeof (float) * 2 * pv->hop_res);
+	  CHECK_MALLOC (fl_out, "pv_complex_resample");
+	  hop_res0 = pv->hop_res;
 	}
     }
 
+  srdata.input_frames  = pv->hop_syn;
+  srdata.output_frames = pv->hop_res;
+  srdata.src_ratio = (double)(pv->hop_res) / (double)(pv->hop_syn);
+  srdata.data_in  = fl_in;
+  srdata.data_out = fl_out;
+
+  // samplerate conversion (time fixed)
+  int i;
+  for (i = 0; i < pv->hop_syn; i ++)
+    {
+      fl_in [i*2 + 0] = (float)(pv->l_out [i]);
+      fl_in [i*2 + 1] = (float)(pv->r_out [i]);
+    }
+  int status;
+  //status = src_simple (&srdata, SRC_SINC_BEST_QUALITY, 2);
+  status = src_simple (&srdata, SRC_SINC_FASTEST, 2);
+  // this works better
+  if (status != 0)
+    {
+      fprintf (stderr, "fail to samplerate conversion\n");
+      exit (1);
+    }
+
+  for (i = 0; i < pv->hop_res; i ++)
+    {
+      left [i]  = (double)(fl_out [i*2 + 0]);
+      right [i] = (double)(fl_out [i*2 + 1]);
+    }
+}
+
+/* play l[n] and r[n] into ao or snd devices
+ * INPUT
+ * OUTPUT
+ *  returned value : number of frames (should be equal to n)
+ */
+int
+pv_complex_play (struct pv_complex *pv,
+		 int n, double *l, double *r)
+{
+  int status = 0;
+  if (pv->flag_out == 0)
+    {
+      status = ao_write (pv->ao, l, r, n);
+      status /= 4; // 2 bytes for 2 channels
+    }
+  else if (pv->flag_out == 1)
+    {
+      status = sndfile_write (pv->sfout, *(pv->sfout_info),
+			      l, r, n);
+    }
+  else
+    {
+      fprintf (stderr, "invalid output device\n");
+    }
+
   return (status);
 }
 
-/* play one hop_in by the phase vocoder:
+/* play the segment of pv->[lr]_out[] for pv->hop_syn
+ * pv->pitch_shift is taken into account, so that 
+ * the output frames are pv->hop_res.
+ * OUTPUT
+ *  returned value : output frames (should be hop_res)
+ */
+int
+pv_complex_play_resample (struct pv_complex *pv)
+{
+  int status;
+
+  if (pv->hop_syn != pv->hop_res)
+    {
+      // samplerate conversion
+      static double *l_resamp = NULL;
+      static double *r_resamp = NULL;
+      static int hop_res0 = 0;
+      if (l_resamp == NULL)
+	{
+	  l_resamp = (double *)malloc (sizeof (double) * pv->hop_res);
+	  r_resamp = (double *)malloc (sizeof (double) * pv->hop_res);
+	  CHECK_MALLOC (l_resamp, "pv_complex_play_resample");
+	  CHECK_MALLOC (r_resamp, "pv_complex_play_resample");
+	  hop_res0 = pv->hop_res;
+	}
+      else if (hop_res0 < pv->hop_res)
+	{
+	  l_resamp =
+	    (double *)realloc (l_resamp, sizeof (double) * pv->hop_res);
+	  r_resamp =
+	    (double *)realloc (r_resamp, sizeof (double) * pv->hop_res);
+	  CHECK_MALLOC (l_resamp, "pv_complex_play_resample");
+	  CHECK_MALLOC (r_resamp, "pv_complex_play_resample");
+	  hop_res0 = pv->hop_res;
+	}
+
+      pv_complex_resample (pv, l_resamp, r_resamp);
+      status = pv_complex_play (pv, pv->hop_res, l_resamp, r_resamp);
+    }
+  else
+    {
+      status = pv_complex_play (pv, pv->hop_syn, pv->l_out, pv->r_out);
+    }
+
+  return (status);
+}
+
+
+/* play one hop_syn by the phase vocoder:
  * phase vocoder by complex arithmetics with fixed hops.
  *   t_i - s_i = u_i - u_{i-1} = hop
  *   where s_i and t_i are the times for two analysis FFT
  *   and u_i is the time for the synthesis FFT at step i
  * Reference: M.Puckette (1995)
  * INPUT
- *  pv : struct pv_complex_data
+ *  pv : struct pv_complex
  *  cur : current frame to play.
  *        you have to increment this by yourself.
  *  pv->flag_lock : 0 == no phase lock
  *                  1 == loose phase lock
  * OUTPUT (returned value)
- *  status : output frame.
+ *  status : output frames (should be hop_res)
  */
-long pv_complex_play_step (struct pv_complex_data *pv,
-			   long cur)
+long
+pv_complex_play_step (struct pv_complex *pv,
+		      long cur)
 {
-  int i;
+  static int len = 0;
+  static double *l_fs  = NULL;
+  static double *r_fs  = NULL;
+  static double *l_ft  = NULL;
+  static double *r_ft  = NULL;
+  static double *l_tmp = NULL;
+  static double *r_tmp = NULL;
 
-  double *l_fs = NULL;
-  double *r_fs = NULL;
-  l_fs = (double *)malloc (pv->len * sizeof (double));
-  r_fs = (double *)malloc (pv->len * sizeof (double));
-  CHECK_MALLOC (l_fs, "pv_complex_play_step");
-  CHECK_MALLOC (r_fs, "pv_complex_play_step");
+  if (l_fs == NULL)
+    {
+      l_fs = (double *)malloc (pv->len * sizeof (double));
+      r_fs = (double *)malloc (pv->len * sizeof (double));
+      CHECK_MALLOC (l_fs, "pv_complex_play_step");
+      CHECK_MALLOC (r_fs, "pv_complex_play_step");
+
+      l_ft = (double *)malloc (pv->len * sizeof (double));
+      r_ft = (double *)malloc (pv->len * sizeof (double));
+      CHECK_MALLOC (l_ft, "pv_complex_play_step");
+      CHECK_MALLOC (r_ft, "pv_complex_play_step");
+
+      l_tmp = (double *)malloc (pv->len * sizeof (double));
+      r_tmp = (double *)malloc (pv->len * sizeof (double));
+      CHECK_MALLOC (l_tmp, "pv_complex_play_step");
+      CHECK_MALLOC (r_tmp, "pv_complex_play_step");
+
+      len = pv->len;
+    }
+  else if (len < pv->len)
+    {
+      l_fs = (double *)realloc (l_fs, pv->len * sizeof (double));
+      r_fs = (double *)realloc (r_fs, pv->len * sizeof (double));
+      CHECK_MALLOC (l_fs, "pv_complex_play_step");
+      CHECK_MALLOC (r_fs, "pv_complex_play_step");
+
+      l_ft = (double *)realloc (l_ft, pv->len * sizeof (double));
+      r_ft = (double *)realloc (r_ft, pv->len * sizeof (double));
+      CHECK_MALLOC (l_ft, "pv_complex_play_step");
+      CHECK_MALLOC (r_ft, "pv_complex_play_step");
+
+      l_tmp = (double *)realloc (l_tmp, pv->len * sizeof (double));
+      r_tmp = (double *)realloc (r_tmp, pv->len * sizeof (double));
+      CHECK_MALLOC (l_tmp, "pv_complex_play_step");
+      CHECK_MALLOC (r_tmp, "pv_complex_play_step");
+
+      len = pv->len;
+    }
 
   long status;
-  // read the starting frame (cur)
+  /* read starting data [cur, cur + len]
+   * ==> FFT ==> fs[len] 
+   */
   status = read_and_FFT_stereo (pv, cur, l_fs, r_fs);
   if (status != pv->len)
     {
-      free (l_fs);
-      free (r_fs);
-
       return 0; // no output
     }
 
-  double *l_ft = NULL;
-  double *r_ft = NULL;
-  l_ft = (double *)malloc (pv->len * sizeof (double));
-  r_ft = (double *)malloc (pv->len * sizeof (double));
-  CHECK_MALLOC (l_ft, "pv_complex_play_step");
-  CHECK_MALLOC (r_ft, "pv_complex_play_step");
-
-  // read the terminal frame (cur + hop_syn)
+  /* read terminal data [cur + hop_syn, cur + hop_syn + len]
+   * ==> FFT ==> ft[len]
+   */
   status = read_and_FFT_stereo (pv, cur + pv->hop_syn, l_ft, r_ft);
   if (status != pv->len)
     {
-      free (l_fs);
-      free (r_fs);
-      free (l_ft);
-      free (r_ft);
-
       return 0; // no output
     }
 
+  // check zero
   int flag_left_cur;
   int flag_right_cur;
   if (check_zero (pv->len, l_fs) == 0 ||
@@ -417,7 +500,6 @@ long pv_complex_play_step (struct pv_complex_data *pv,
     {
       flag_left_cur = 1; // active
     }
-
   if (check_zero (pv->len, r_fs) == 0 ||
       check_zero (pv->len, r_ft) == 0)
     {
@@ -429,14 +511,10 @@ long pv_complex_play_step (struct pv_complex_data *pv,
     }
 
 
-  double *l_tmp = NULL;
-  double *r_tmp = NULL;
-  l_tmp = (double *)malloc (pv->len * sizeof (double));
-  r_tmp = (double *)malloc (pv->len * sizeof (double));
-  CHECK_MALLOC (l_tmp, "pv_complex_play_step");
-  CHECK_MALLOC (r_tmp, "pv_complex_play_step");
-
-
+  /* phase vocoder process
+   * fs[len] and ft[len] ==> superimposing out[hop_syn, hop_syn + len]
+   */
+  int i;
   // left channel
   if (flag_left_cur == 1)
     {
@@ -479,9 +557,6 @@ long pv_complex_play_step (struct pv_complex_data *pv,
 	  apply_invFFT_mono (pv, l_tmp, pv->window_scale, pv->l_out);
 	}
     }
-  free (l_fs);
-  free (l_ft);
-  free (l_tmp);
 
 
   // right channel
@@ -525,16 +600,17 @@ long pv_complex_play_step (struct pv_complex_data *pv,
 	  apply_invFFT_mono (pv, r_tmp, pv->window_scale, pv->r_out);
 	}
     }
-  free (r_fs);
-  free (r_ft);
-  free (r_tmp);
 
 
-  // output
+  /* output
+   * out[0, hop_syn] ==> resample into hop_res ==> ao derive or snd file
+   */
   status = pv_complex_play_resample (pv);
 
 
-  /* shift [lr]_out by hop_syn */
+  /* shift
+   * out[hop_syn, hop_syn + len] ==> out[0, len]
+   */
   for (i = 0; i < pv->len; i ++)
     {
       pv->l_out [i] = pv->l_out [i + pv->hop_syn];
@@ -565,18 +641,15 @@ long pv_complex_play_step (struct pv_complex_data *pv,
  *  pitch_shift : in the unit of half-note
  */
 void pv_complex (const char *file, const char *outfile,
-		 double rate, double pitch_shift,
-		 long len, long hop_syn,
-		 int flag_window,
-		 int flag_lock)
+		  double rate, double pitch_shift,
+		  long len, long hop_syn,
+		  int flag_window,
+		  int flag_lock)
 {
-  long hop_ana;
-  long hop_res;
-  hop_res = (long)((double)hop_syn * pow (2.0, - pitch_shift / 12.0));
-  hop_ana = (long)((double)hop_res * rate);
+  long hop_res = (long)((double)hop_syn * pow (2.0, - pitch_shift / 12.0));
+  long hop_ana = (long)((double)hop_res * rate);
 
-  struct pv_complex_data *pv = NULL;
-  pv = pv_complex_init (len, hop_syn, flag_window);
+  struct pv_complex *pv = pv_complex_init (len, hop_syn, flag_window);
   pv->hop_res = hop_res;
   pv->hop_ana = hop_ana;
   //pv->pitch_shift = pitch_shift;
@@ -601,7 +674,7 @@ void pv_complex (const char *file, const char *outfile,
   SF_INFO sfout_info;
   if (outfile == NULL)
     {
-      ao = ao_init_16_stereo (sfinfo.samplerate);
+      ao = ao_init_16_stereo (sfinfo.samplerate, 1 /* verbose */);
       pv_complex_set_output_ao (pv, ao);
     }
   else
@@ -622,9 +695,8 @@ void pv_complex (const char *file, const char *outfile,
   long cur;
   for (cur = 0; cur < (long)sfinfo.frames; cur += pv->hop_ana)
     {
-      long status;
-      status = pv_complex_play_step (pv, cur);
-      if (status < pv->hop_syn)
+      long len_play = pv_complex_play_step (pv, cur);
+      if (len_play < pv->hop_res)
 	{
 	  break;
 	}
